//+------------------------------------------------------------------+
//|                                                  SettingsBar.mqh |
//+------------------------------------------------------------------+

#property copyright "TheXpert"
#property link      "theforexpert@gmail.com"

#include "Assert.mqh"

#include "IntSetting.mqh"
#include "DoubleSetting.mqh"
#include "StringSetting.mqh"
#include "SelectionSetting.mqh"
#include "BoolSetting.mqh"

#include "BlockMover.mqh"
#include "BlockHider.mqh"

class SettingsBar
{
public:
   SettingsBar();
   ~SettingsBar();
   
   void AddIntSetting(string settingName, int settingID, int value, int step = 1, int min = INT_MIN, int max = INT_MAX);
   void AddDoubleSetting(string settingName, int settingID, double value, double step = 1, double min = -DBL_MAX, double max = DBL_MAX);
   void AddStringSetting(string settingName, int settingID, string value);
   void AddSelectionSetting(string settingName, int settingID, int valueID, const SelectionTable& ids);
   void AddBoolSetting(string settingName, int settingID, bool value);
   
   void OnChartEvent(const int id, const long& lparam, const double& dparam, const string& sparam);
   
   void SetTitle(string newTitle);
   
private:
   void DrawBar();
   void Hide();
   void DrawNextButton();
   void DrawPrevButton();
   void DrawBarArea();
   void DrawCurrentSetting();
   void HideCurrentSetting();

private:
   DrawSetting* m_Settings[];
   BlockMover m_Mover;
   BlockHider m_Hider;
   
   string m_NextButton;
   string m_PrevButton;
   string m_BarArea;
   string m_BarTitle;
   
   int m_UID;
   int m_X;
   int m_Y;
   int m_Chart;
   int m_Window;
   
   int m_CurrentIndex;
   
   string m_Title;
};

SettingsBar::SettingsBar(void)
{
   m_UID = GetUID();
   m_NextButton = "NextSetting" + string(m_UID);
   m_PrevButton = "PrevSetting" + string(m_UID);
   m_BarArea = "BarArea" + string(m_UID);
   m_BarTitle = "BarTitle" + string(m_UID);
   
   m_CurrentIndex = 0;
   
   m_Mover.Init(20, 20, 0);
}

void SettingsBar::OnChartEvent(const int id, const long& lparam, const double& dparam, const string& sparam)
{
   m_Mover.OnChartEvent(id, lparam, dparam, sparam);
   m_Hider.OnChartEvent(id, lparam, dparam, sparam);

   if (sparam == m_Mover.Name() && id == CHARTEVENT_CUSTOM + BLOCK_MOVER_EVENT)
   {
      m_X = int(lparam) + 31;
      m_Y = int(dparam);
      m_Chart = 0;
      m_Window = ObjectFind(m_Chart, m_Mover.Name());
      
      DrawBar();
      m_Hider.Draw(m_X - 16, m_Y, m_Chart, m_Window);
   }

   if (sparam == m_Hider.Name() && id == CHARTEVENT_CUSTOM + BLOCK_HIDER_EVENT)
   {
      DrawBar();
   }
   
   if (id == CHARTEVENT_OBJECT_DELETE)
   {
      if (sparam == m_BarArea)
      {
         Hide();
         DrawBar();
      }
      else if (sparam == m_BarTitle)
      {
         DrawBar();
      }
      else if (!m_Hider.Minimized())
      {
         if (sparam == m_NextButton)
         {
            DrawNextButton();
         }
         else if (sparam == m_PrevButton)
         {
            DrawPrevButton();
         }
      }
   }

   if (id == CHARTEVENT_OBJECT_CLICK)
   {
      if (sparam == m_NextButton)
      {
         HideCurrentSetting();
         
         m_CurrentIndex++;
         if (m_CurrentIndex >= ArraySize(m_Settings))
         {
            m_CurrentIndex = 0;
         }
         DrawBar();
      }
      else if (sparam == m_PrevButton)
      {
         HideCurrentSetting();

         m_CurrentIndex--;
         if (m_CurrentIndex < 0)
         {
            m_CurrentIndex = int(MathMax(0, ArraySize(m_Settings) - 1));
         }
         DrawBar();
      }
   }
   
   int size = ArraySize(m_Settings);
   if (size > 0)
   {
      if (m_Settings[m_CurrentIndex] != NULL)
      {
         m_Settings[m_CurrentIndex].OnChartEvent(id, lparam, dparam, sparam);
      }
   }
}

SettingsBar::DrawBar()
{
   DrawBarArea();
   MoveObject(m_Chart, m_BarArea, m_X, m_Y, 0);
   MoveObject(m_Chart, m_BarTitle, m_X, m_Y - 15, 0);

   if (!m_Hider.Minimized())
   {
      ObjectSetInteger(m_Chart, m_BarArea, OBJPROP_YSIZE, 60);
      
      DrawNextButton();
      MoveObject(m_Chart, m_NextButton, m_X + 5, m_Y + 5, 0);

      DrawPrevButton();
      MoveObject(m_Chart, m_PrevButton, m_X + 5, m_Y + 35, 0);

      DrawCurrentSetting();      
   }
   else
   {
      Hide();   
   }
}

void SettingsBar::Hide()
{
   ObjectSetInteger(m_Chart, m_BarArea, OBJPROP_YSIZE, 15);
   ObjectDelete(m_Chart, m_NextButton);
   ObjectDelete(m_Chart, m_PrevButton);
   
   HideCurrentSetting();
}

SettingsBar::~SettingsBar()
{
   ObjectDelete(m_Chart, m_NextButton);
   ObjectDelete(m_Chart, m_PrevButton);
   ObjectDelete(m_Chart, m_BarArea);
   ObjectDelete(m_Chart, m_BarTitle);
   
   int size = ArraySize(m_Settings);
   for (int i = 0; i < size; i++)
   {
      if (m_Settings[i] != NULL)
      {
         delete m_Settings[i];
      }
   }
}

void SettingsBar::DrawNextButton()
{
   CreateObject(m_Chart, m_Window, m_NextButton, OBJ_BUTTON, CharToString(217));
   ObjectSetString(m_Chart, m_NextButton, OBJPROP_FONT, "Wingdings");
   ObjectSetInteger(m_Chart, m_NextButton, OBJPROP_XSIZE, 20);
   ObjectSetInteger(m_Chart, m_NextButton, OBJPROP_YSIZE, 20);
   ObjectSetInteger(m_Chart, m_NextButton, OBJPROP_STATE, false);
}

void SettingsBar::DrawPrevButton()
{
   CreateObject(m_Chart, m_Window, m_PrevButton, OBJ_BUTTON, CharToString(218));
   ObjectSetString(m_Chart, m_PrevButton, OBJPROP_FONT, "Wingdings");
   ObjectSetInteger(m_Chart, m_PrevButton, OBJPROP_XSIZE, 20);
   ObjectSetInteger(m_Chart, m_PrevButton, OBJPROP_YSIZE, 20);
   ObjectSetInteger(m_Chart, m_PrevButton, OBJPROP_STATE, false);
}

void SettingsBar::DrawBarArea()
{
   CreateObject(m_Chart, m_Window, m_BarArea, OBJ_EDIT, "");
   ObjectSetInteger(m_Chart, m_BarArea, OBJPROP_READONLY, true);
   ObjectSetInteger(m_Chart, m_BarArea, OBJPROP_XSIZE, 300);

   CreateObject(m_Chart, m_Window, m_BarTitle, OBJ_LABEL, m_Title);
}

void SettingsBar::AddIntSetting(string settingName, int settingID, int value, int step = 1, int min = INT_MIN, int max = INT_MAX)
{
   int size = ArraySize(m_Settings);
   ArrayResize(m_Settings, size + 1);
   
   IntSetting* newSetting = new IntSetting;
   newSetting.Init(settingName, settingID, GetUID(), value, step, min, max);
   
   m_Settings[size] = newSetting;
   
   if (size == 0) DrawBar();
}

void SettingsBar::AddDoubleSetting(string settingName, int settingID, double value, double step = 1, double min = -DBL_MAX, double max = DBL_MAX)
{
   int size = ArraySize(m_Settings);
   ArrayResize(m_Settings, size + 1);
   
   DoubleSetting* newSetting = new DoubleSetting;
   newSetting.Init(settingName, settingID, GetUID(), value, step, min, max);
   
   m_Settings[size] = newSetting;

   if (size == 0) DrawBar();
}

void SettingsBar::AddStringSetting(string settingName, int settingID, string value)
{
   int size = ArraySize(m_Settings);
   ArrayResize(m_Settings, size + 1);
   
   StringSetting* newSetting = new StringSetting;
   newSetting.Init(settingName, settingID, GetUID(), value);
   
   m_Settings[size] = newSetting;

   if (size == 0) DrawBar();
}

void SettingsBar::AddSelectionSetting(string settingName, int settingID, int valueID, const SelectionTable& ids)
{
   int size = ArraySize(m_Settings);
   ArrayResize(m_Settings, size + 1);
   
   SelectionSetting* newSetting = new SelectionSetting;
   newSetting.Init(settingName, settingID, GetUID(), valueID, ids);
   
   m_Settings[size] = newSetting;

   if (size == 0) DrawBar();
}

void SettingsBar::AddBoolSetting(string settingName, int settingID, bool value)
{
   int size = ArraySize(m_Settings);
   ArrayResize(m_Settings, size + 1);
   
   BoolSetting* newSetting = new BoolSetting;
   newSetting.Init(settingName, settingID, GetUID(), value);
   
   m_Settings[size] = newSetting;

   if (size == 0) DrawBar();
}

void SettingsBar::DrawCurrentSetting(void)
{
   int size = ArraySize(m_Settings);
   if (size > 0)
   {
      if (m_Settings[m_CurrentIndex] != NULL)
      {
         ISettingEvents* ptr = m_Settings[m_CurrentIndex];
         ptr.Draw(m_X + 30, m_Y, m_Chart, m_Window);
      }
   }
}

void SettingsBar::HideCurrentSetting(void)
{
   int size = ArraySize(m_Settings);
   if (size > 0)
   {
      if (m_Settings[m_CurrentIndex] != NULL)
      {
         ISettingEvents* ptr = m_Settings[m_CurrentIndex];
         ptr.Hide();
      }
   }
}

void SettingsBar::SetTitle(string newTitle)
{
   m_Title = newTitle;
}